/* START OF radiogroup.c -------------------------------------------------------
 *
 *	A slide bar widget.
 *
 *	---
 *	Written by David Eggleston (2006) <deggleston@users.sourceforge.net>
 *	for the 'BeebEm' Acorn BBC Model B, Integra-B, Model B Plus and
 *	Master 128 emulator.
 *
 *	This file is part of 'Economy GUI' and may be copied only under the
 *	terms of either the GNU General Public License (GPL) or Dr. David
 *	Alan Gilbert's BeebEm license.
 *
 *	For more details please visit:
 *
 *	http://www.gnu.org/copyleft/gpl.html
 *	---
 */


#if HAVE_CONFIG_H
#       include <config.h>
#endif


#include "include/log.h"

#include "include/gui_functions.h"

#include "include/gui_slidebar.h"
#include "include/gui_slidebar_private.h"

#include "include/gui_button.h"
#include "include/gui_box.h"
#include "include/gui_label.h"

#include "include/gui_window.h"
#include "include/gui_window_private.h"

#include "include/gui_widget.h"
#include "include/gui_widget_private.h"

#include "include/gui_widget_shared.h"

#include "include/sdl.h"

#include <SDL.h>


/* Callbacks:
 */

static void Callback_Destroy(EG_Widget *widget_ptr)
{
	EG_SlideBar *slidebar_ptr;
	EG_Widget *tmp_widget_ptr;
	int i;

	/* If widget is actually just a NULL, then do nothing but log error.
	 */
	if (widget_ptr==NULL){
		EG_Log(EG_LOG_ERROR, dL"Tried to Destroy Slide Bar with"
		 " widget_ptr=NULL", dR);
              return;
	}

	/* If widget is not a SlideBar, then cannot delete composite children.
	 */
	if (EG_Widget_GetType(widget_ptr) != EG_Widget_Type_SlideBar){
		EG_Log(EG_LOG_ERROR, dL"EG_SlideBar Destroy callback called"
		 " for none EG_SlideBar widget.", dR);
	}else{
		/* Unlink composite children and slidebar payload.
		 */
		if ( (slidebar_ptr= (EG_SlideBar*)
		 EG_Widget_GetPayload(widget_ptr)) != NULL){
        	}
		
		// [TODO] free composite widgets.

		/* Free slide bar object instance.
		 */
		EG_Free(slidebar_ptr);
	}
  
	/* Free EG_Widget struct.
	 */
	EG_Widget_Free(widget_ptr);
}

static EG_BOOL Callback_Paint(EG_Widget *widget_ptr, SDL_Rect area)
{
        EG_SlideBar *slidebar_ptr;
	SDL_Rect *tmp_area;
	int i;

	/* Loose compiler warning.
	 */
	tmp_area = &area;

	/* If asked to paint, paint the children.
	 */
        EG_SLIDEBAR_GET_STRUCT_PTR(widget_ptr, slidebar_ptr, EG_FALSE);


	// [TODO] Repaint composite widgets.

	/* Return success a painting.
	 */
	return(EG_TRUE);
}

static EG_BOOL Callback_Enabled(EG_Widget *widget_ptr, EG_BOOL is_enabled)
{
	EG_SlideBar *slidebar_ptr;
	EG_Widget *tmp_widget_ptr;
	int i;

	CHECK_EG_WIDGET_IS_VALID(widget_ptr, EG_FALSE);
	EG_SLIDEBAR_GET_STRUCT_PTR(widget_ptr, slidebar_ptr, EG_FALSE);

	/* Change state of Slide Bar widget.
	 */
	(void) EG_Shared_UpdateEnabledState(widget_ptr, is_enabled);

	/* Reflect change with children widgets too.
	 */

	// [TODO]

	return(EG_TRUE);
}       

/* When Slide Bar is attached to a window, make sure all child composite widgets
 * are to.
 */
static void Callback_Attach(EG_Widget *widget_ptr, EG_StringHash attach_to_type
 , void *attach_to_ptr, EG_BOOL attached)
{
	EG_SlideBar *slidebar_ptr;
	EG_Window *window_ptr;
	EG_StringHash tmp_attach_to_type;
	int i;

	/* Loose compiler warning.
	 */
//	tmp_attach_to_type = attach_to_type;

	/* We only need to add the buttons to the window when attaching, we do
	 * nothing when detatching.
	 */
	if (attached != EG_TRUE)
		return;

	CHECK_EG_WIDGET_IS_VALID_VOID(widget_ptr);
	EG_SLIDEBAR_GET_STRUCT_PTR_VOID(widget_ptr, slidebar_ptr);

	/* Get parent window.
	 */
	window_ptr = (EG_Window*) attach_to_ptr;
	CHECK_EG_WINDOW_IS_VALID_VOID(window_ptr);

	/* Add the composite child widgets to window.
	 */

	// [TODO]
}


/* Private functions:
 */

static void InitializePayload(EG_SlideBar *slidebar_ptr)
{
	int i;
}

static void InitializeWidget(EG_Widget *widget_ptr)
{
	/* Initialize callbacks:
	 */
      	(void) EG_Widget_SetCallback_Destroy(widget_ptr, Callback_Destroy);

        (void) EG_Widget_SetCallback_Paint(widget_ptr, Callback_Paint);

        (void) EG_Widget_SetCallback_SDL_Event(widget_ptr
	 , EG_Callback_Generic_SDL_Event);

        (void) EG_Widget_SetCallback_Enabled(widget_ptr, Callback_Enabled);

        (void) EG_Widget_SetCallback_GotFocus(widget_ptr
	 , EG_Callback_Generic_GotFocus_NoSupport);

        (void) EG_Widget_SetCallback_LostFocus(widget_ptr
	 , EG_Callback_Generic_LostFocus);

        (void) EG_Widget_SetCallback_Attach(widget_ptr, Callback_Attach);

        /* Initialize state:
         */

	/* Only composite widgets can get focus.
	 */
	(void) EG_Widget_SetCanGetFocusToggle(widget_ptr, EG_FALSE);

	/* Initialize composite widgets.
	 */


}


/* Public functions:
 */

EG_Widget* EG_SlideBar_Create(const char *name_ptr)
{
        EG_SlideBar *slidebar_ptr;
        EG_Widget *widget_ptr;
	void *ptr;

        SHARED__ALLOC_PAYLOAD_STRUCT(ptr, EG_SlideBar
         , "Unable to malloc EG_SlideBar struct");
	slidebar_ptr = (EG_SlideBar*) ptr;

        SHARED__CREATE_NEW_EG_WIDGET(widget_ptr, name_ptr
	 , EG_Widget_Type_SlideBar, slidebar_ptr);

        SHARED__ATTACH_PAYLOAD_TO_WIDGET(widget_ptr, slidebar_ptr);

	InitializePayload(slidebar_ptr);
	InitializeWidget(widget_ptr);

        return(widget_ptr);
}


/* H = 
 *
 */
