/* START OF slidebar_private.h --------------------------------------------------
 *
 *	A slide bar widget.
 * 
 *	---
 *	Written by David Eggleston (2006) <deggleston@users.sourceforge.net>
 *	for the 'BeebEm' Acorn BBC Model B, Integra-B, Model B Plus and
 *	Master 128 emulator.
 *
 *	This file is part of 'Economy GUI' and may be copied only under the 
 *	terms of either the GNU General Public License (GPL) or Dr. David
 *	Alan Gilbert's BeebEm license.
 *
 *	For more details please visit:
 *
 *	http://www.gnu.org/copyleft/gpl.html
 *	---
 */


#ifndef _DW_SLIDEBAR_PRIVATE_H_
#define _DW_SLIDEBAR_PRIVATE_H_


#if HAVE_CONFIG_H
#       include <config.h>
#endif

#include "include/types.h"

#include "include/gui_widget.h"
#include "include/gui_window.h"

#include <SDL.h>


/* Get EG_SlideBar struct.
 */

#define EG_SLIDEBAR_GET_STRUCT_PTR(w, v, r)  \
        if (w == NULL ){ \
                EG_Log(EG_LOG_ERROR, dL"EG_Widget is NULL.", dR); \
                return( r ); \
        } \
        if (EG_Widget_GetType( w ) != EG_Widget_Type_SlideBar){ \
                EG_Log(EG_LOG_ERROR, dL"EG_Widget payload is not of type EG_SlideBar.", dR); \
                return( r ); \
        } \
        if ( (v = (EG_SlideBar*) EG_Widget_GetPayload( w )) == NULL){ \
                EG_Log(EG_LOG_ERROR, dL"EG_Widget payload is NULL.", dR); \
                return(r); \
        } \
        EG_ASSERT_MALLOC( v );

#define EG_SLIDEBAR_GET_STRUCT_PTR_VOID(w, v)  \
        if (w == NULL ){ \
                EG_Log(EG_LOG_ERROR, dL"EG_Widget is NULL.", dR); \
                return; \
        } \
        if (EG_Widget_GetType( w ) != EG_Widget_Type_SlideBar){ \
                EG_Log(EG_LOG_ERROR, dL"EG_Widget payload is not of type EG_SlideBar.", dR); \
                return; \
        } \
        if ( (v = (EG_SlideBar*) EG_Widget_GetPayload( w )) == NULL){ \
                EG_Log(EG_LOG_ERROR, dL"EG_Widget payload is NULL.", dR); \
                return; \
        } \
        EG_ASSERT_MALLOC( v );



/* EG_Widget payload:
 *
 * |------- ]h
 * |      | |B X---- A
 * |     V| +y | b
 * |      | |  X---- A
 *-+-Y    | ]f
 * |X    /
 * |X   /
 *H|X  / 
 * |  /
 * | /
 * |/
 *
 * B = V - h - f
 * Y = (H-V)/A x Y, Y = 0 to H-V
 * y = A/H x Y
 * A = B-b
 * b = B x V/H >= 16 x scale
 *
 * H = Height of scrollable area.
 * Y = Top scanline that is visible.
 * V = Amount of height visible onscreen.
 * B = Area of scrollbar.
 * h = Header area of scrollbar (the up arrow area).
 * f = Footer area of scrollbar (the down arrow area).
 * b = Height of troff on scrollbar.
 * A = Area remaining in scrollbar area (B) minus the troff (b).
 * y = Top scanline of troff in scrollbar area.
 *
 *
 *
 */

typedef struct{
	EG_Widget *button1, *button2, *slider, *troff;

	float H, Y, V, B, h, f, b, A, y;


}EG_SlideBar;
#define EG_AsSlideBar(n) ((EG_SlideBar*) n)

/* Private and friend functions:
 */

/* END OF slidebar_private.h ----------------------------------------------------
 */
#endif



